'''
12/14/2019
Description:
A Tool (UI Included) that help animators use character rigs 
Created by characterrigs.com

to do:
    def updateIconName(self)
    def updateIconPic(self)
    
    subclass a new QGraphicsScene and overwrite the mouse click even
'''
import sys

if 'PySide2' in sys.modules:
    sys.modules['PySide'] = sys.modules['PySide2']
    sys.modules['shiboken'] = sys.modules['shiboken2']

    import pyside2uic
    sys.modules['pysideuic'] = sys.modules['pyside2uic']
    sys.modules['PySide.QtGui'] = sys.modules['PySide2.QtGui']
    sys.modules['PySide.QtCore'] = sys.modules['PySide2.QtCore']
    sys.modules["PySide.QtGui"].__dict__.update(sys.modules["PySide2.QtWidgets"].__dict__)

    from PySide2 import QtWidgets
    from PySide import QtGui, QtCore

    setattr(sys.modules["PySide.QtGui"], "QApplication", sys.modules["PySide2.QtWidgets"].QApplication)
    setattr(sys.modules["PySide.QtGui"].QApplication, "UnicodeUTF8", -1)
    setattr(sys.modules["PySide.QtGui"], "QSortFilterProxyModel", sys.modules["PySide2.QtCore"].QSortFilterProxyModel)
    setattr(sys.modules["PySide.QtGui"].QHeaderView, "setResizeMode", sys.modules["PySide2.QtWidgets"].QHeaderView.setSectionResizeMode)

from maya import cmds
from maya import mel
from maya import OpenMayaUI as omui 
import maya.api.OpenMaya as om

from PySide.QtCore import * 
from PySide.QtGui import * 
from shiboken import wrapInstance
from functools import partial
import math
import json
import os
import pprint #Pretty Print 
import ast #Convert string representation of list to list in Python


mayaMainWindowPtr = omui.MQtUtil.mainWindow()
mayaMainWindow = wrapInstance(long(mayaMainWindowPtr), QWidget)

class BaseCallBack(object):
    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.args = args
        self.kwargs = kwargs

class UndoCallback(BaseCallBack):
    def __call__(self, *args):
        cmds.undoInfo(openChunk=1)
        try:
            return self.func(*self.args, **self.kwargs)
        finally:
            cmds.undoInfo(closeChunk=1)

def dump(data):
    '''
    encodes python data as json data
    '''
    if isinstance(data, dict):
        return json.dumps(data, sort_keys=True, indent=4)
    else:
        raise TypeError('{0} must be dict() type'.format(data))

def saveJson(data, filepath=None):
    f = open(filepath, 'w')
    new_data = dump(data)
    f.write(new_data)
    f.close()
    return filepath

def loadJson(filepath):
    f = open(filepath, 'r')
    data = json.loads(f.read())
    f.close()
    return data

def showDockUI(reloadUI=False):
    '''make dockable ui'''
    if cmds.dockControl('Character_Tool_Dock', q=True, ex=True):
        cmds.deleteUI('Character_Tool_Dock')  

    mayaWindow = cmds.window(title='Character Tool v2.0')
    mayaLayout = cmds.paneLayout(configuration='single')
    
    if reloadUI:
        [obj.deleteLater() for obj in qApp.topLevelWidgets() if obj.objectName() == "Character_Tool"]
        characterToolObj = CharacterTool()
    else:
        qNames = [obj.objectName() for obj in qApp.topLevelWidgets()]
        if "Character_Tool" not in qNames: characterToolObj = CharacterTool()

    cmds.dockControl("Character_Tool_Dock", l='Character Tool', w=450, area = 'left', content=mayaWindow, allowedArea=['left', 'right'])
    cmds.control('Character_Tool', e=True, p=mayaLayout)


#CharacterTool object
class CharacterTool(QWidget):
    def __init__(self, *args, **kwargs):        
        super(CharacterTool, self).__init__(*args, **kwargs)
        
        self.tmpAttr = {} #a data container used to copy and paste attr
        self.pickerDic = {} #a dta container used to store the picker data

        #Parent widget under Maya main window        
        self.setParent(mayaMainWindow)        
        self.setWindowFlags(Qt.Window)   
        
        #Set the object name     
        self.setObjectName('Character_Tool')        
        self.setWindowTitle('Character Tool v2.0')        
        self.setGeometry(50, 200, 500, 870) 

        self.path = os.path.dirname(os.path.realpath(__file__))
        self.iconDir = os.path.join(self.path, 'Icons')

        self.createLayout()
        self.editPickerLayout()
        self.newTabLayout()
        self.newPickerLayout()
        self.mirrorPickerLayout()

        self.customizeWidget()
        self.connectWidgets()

        #self.updateCtrlTree() self.updateCtrlPicker() done in load characters
        self.loadCharacters()
        self.getVersionInfo()

            
    def connectWidgets(self):
        self.alphaSlider.valueChanged.connect(self.changeUIOpacity)
        self.refreshBttn.clicked.connect(self.loadCharacters)
        self.modeBttn.clicked.connect(self.modeSwitch)

        self.characterCombo.currentIndexChanged.connect(self.getVersionInfo)
        self.characterCombo.currentIndexChanged.connect(self.updateCtrlTree)
        self.characterCombo.currentIndexChanged.connect(self.updateCtrlPicker)
        
        self.characterCombo.currentIndexChanged.connect(self.updateIKFKList)
        self.characterCombo.currentIndexChanged.connect(self.updateSpaceCtrlList)
        self.characterCombo.currentIndexChanged.connect(self.updateSpaceList)

        #---
        self.ctrlTree.clicked.connect(self.selectTreeCtrls)

        self.showVisBttn.clicked.connect(partial(self.showHideAllCtrls, 1))
        self.hideVisBttn.clicked.connect(partial(self.showHideAllCtrls, 0))
        self.isolateVisBttn.clicked.connect(self.isolateSelectedCtrls)

        #---
        self.ikfkList.clicked.connect(UndoCallback(self.doikfkSwitch))
        self.spaceCtrlList.clicked.connect(self.updateSpaceList)
        self.spaceCtrlList.currentRowChanged.connect(self.updateSpaceList)
        self.spaceList.clicked.connect(UndoCallback(self.spaceSwitch))

        #---
        self.keyAllBttn.clicked.connect(UndoCallback(self.keyAllCtrls))
        self.resetAllBttn.clicked.connect(UndoCallback(self.resetAllCtrls))
        self.resetSelBttn.clicked.connect(UndoCallback(self.resetSelectedCtrls))

        self.copyKeyBttn.clicked.connect(self.copyAttr)
        self.pasteKeyBttn.clicked.connect(UndoCallback(self.pasteAttr))
        self.deleteKeyBttn.clicked.connect(UndoCallback(self.deleteKey))
        self.cleanKeyBttn.clicked.connect(UndoCallback(self.cleanObjAnimationCurve))

        #------------------
        self.editPickerBttn.clicked.connect(self.editPickerWidget.show)
        self.newPickerBttn.clicked.connect(self.newPickerWidget.show)
        self.mirrorPickerBttn.clicked.connect(self.mirrorPickerWidget.show)
        self.newTabBttn.clicked.connect(self.newTabWidget.show)

        self.ctrlTab.currentChanged.connect(self.getCurrentTab)

        #--
        self.doMirrorPickerBttn.clicked.connect(self.mirrorPickers)

        self.posXSpin.valueChanged.connect(self.setIconPos)
        self.posYSpin.valueChanged.connect(self.setIconPos)

        self.movePickerUpBttn.clicked.connect(partial(self.offsetIconPos, "up"))
        self.movePickerDownBttn.clicked.connect(partial(self.offsetIconPos, "down"))
        self.movePickerLeftBttn.clicked.connect(partial(self.offsetIconPos, "left"))
        self.movePickerRightBttn.clicked.connect(partial(self.offsetIconPos, "right"))

        self.allAlignUpBttn.clicked.connect(partial(self.alignIconPos, "up"))
        self.allAlignLowBttn.clicked.connect(partial(self.alignIconPos, "down"))
        self.allAlignLeftBttn.clicked.connect(partial(self.alignIconPos, "left"))
        self.allAlignRightBttn.clicked.connect(partial(self.alignIconPos, "right"))

        self.loadCtrlBttn.clicked.connect(partial(self.loadSelected, self.newCtrlsLine))
        self.createTabBttn.clicked.connect(self.createNewPickerTab)
        self.createPickerBttn.clicked.connect(self.createNewPicker)
        
        #--
        self.browsePathBttn.clicked.connect(self.setNewExportPath)
        self.openPathBttn.clicked.connect(self.openFileFolder)

        #--
        self.savePickerBttn.clicked.connect(self.savePickerDicInScene)
        self.exportPickerBttn.clicked.connect(self.exportPickerDic)
        self.importPickerBttn.clicked.connect(self.importPickerDic)

        self.printPickerBttn.clicked.connect(self.printPickerDic)
        self.deletePickerBttn.clicked.connect(self.deletePickers)
        self.deleteTabBttn.clicked.connect(self.deletePickersTab)

        self.editCtrlBttn.clicked.connect(partial(self.loadSelected, self.ctrlsLine))
        self.editCtrlBttn.clicked.connect(self.updateIconCtrls)
        self.ctrlsLine.returnPressed.connect(self.updateIconCtrls)
        self.pickerNameLine.returnPressed.connect(self.updateIconName)

    def createLayout(self):
        '''
        #add Scroll Area 
        vLayout = QVBoxLayout()
        self.setLayout(vLayout)

        self.scrollArea = QScrollArea()
        self.scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        #self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        vLayout.addWidget(self.scrollArea)

        #Container Widget        
        self.centralWidget = QWidget()
        self.scrollArea.setWidget(self.centralWidget)
        self.centralWidget.resize(410, 1000)
        '''

        #Layout of Container Widget
        mainLayout = QVBoxLayout()
        #self.centralWidget.setLayout(mainLayout)
        self.setLayout(mainLayout)

        self.vSplitter = QSplitter(Qt.Vertical)
        mainLayout.addWidget(self.vSplitter)

        #--1 get character name
        characterWidget = QWidget()
        self.vSplitter.addWidget(characterWidget)
        charLayout = QHBoxLayout()
        characterWidget.setLayout(charLayout)
        charLayout.setContentsMargins(0, 0, 0, 5)

        characterLabel = QLabel('Character:')
        self.characterCombo = QComboBox()
        self.alphaSlider = QSlider(Qt.Horizontal)
        self.refreshBttn = QPushButton()

        try:
            mode = cmds.evaluationManager(mode=True, q=True)
        except:
            mode = ["off"]
        if mode[0] == "parallel":
            self.modeBttn = QPushButton("Parallel")
        elif mode[0] == "off":
            self.modeBttn = QPushButton("   DG    ")

        characterLabel.setMaximumWidth(60)
        self.characterCombo.setMinimumWidth(120)
        self.refreshBttn.setMaximumWidth(50)
        self.alphaSlider.setRange(10,100)
        self.alphaSlider.setValue(100)

        charLayout.addWidget(characterLabel)
        charLayout.addWidget(self.characterCombo)
        charLayout.addWidget(self.alphaSlider)
        charLayout.addWidget(self.refreshBttn)
        charLayout.addWidget(self.modeBttn)
        #charLayout.insertSpacing(2, 150)
        
        #--2 list all character controls
        self.treeWidget = QWidget()
        self.vSplitter.addWidget(self.treeWidget)
        treeLayout = QVBoxLayout()
        self.treeWidget.setLayout(treeLayout)
        treeLayout.setContentsMargins(0, 0, 0, 5)

        #--2.1
        self.ctrlTab = QTabWidget()
        self.ctrlTree = QTreeView()
        self.ctrlTab.addTab(self.ctrlTree, "Ctrl List")
        #self.ctrlTree.setMinimumHeight(350)
        treeLayout.addWidget(self.ctrlTab)

        self.ctrlTree.setHeaderHidden(True)
        self.treeModel = QStandardItemModel()
        self.ctrlTree.setModel(self.treeModel)
        self.ctrlTree.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.ctrlTree.setEditTriggers(QAbstractItemView.NoEditTriggers)

        #--2.2
        treeButtonLayout = QHBoxLayout()
        treeLayout.addLayout(treeButtonLayout)

        self.showVisBttn = QPushButton()
        self.hideVisBttn = QPushButton()
        self.isolateVisBttn = QPushButton()
        [treeButtonLayout.addWidget(bttn) for bttn in [self.showVisBttn, self.hideVisBttn, self.isolateVisBttn]]

        #--
        self.viewMenu = QMenu()
        self.isoSelectAction = QAction("View Selected Controls Only", self)
        self.isoSelectAction.setCheckable(True)
        self.viewMenu.addAction(self.isoSelectAction)

        self.ctrlTab.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ctrlTab.customContextMenuRequested.connect(self.ctrlTabRightClick)

        #--3 ik fk space ctrl tool
        ctrlWidget = QWidget()
        self.vSplitter.addWidget(ctrlWidget)
        ctrlLayout = QVBoxLayout()
        ctrlListLayout = QGridLayout()
        ctrlWidget.setLayout(ctrlLayout)
        ctrlLayout.addLayout(ctrlListLayout)
        ctrlLayout.setContentsMargins(0, 0, 0, 5)

        switchLabel = QLabel('IK/FK Switch:')
        spaceCtrlLabel = QLabel('Space Controls:')
        spaceLabel = QLabel('Space:')

        self.ikfkList = QListWidget()
        self.spaceCtrlList = QListWidget()
        self.spaceList = QListWidget()

        ctrlListLayout.addWidget(switchLabel, 0, 0)
        ctrlListLayout.addWidget(spaceCtrlLabel, 0, 1)
        ctrlListLayout.addWidget(spaceLabel, 0, 2)
        ctrlListLayout.addWidget(self.ikfkList, 1, 0)
        ctrlListLayout.addWidget(self.spaceCtrlList, 1, 1)
        ctrlListLayout.addWidget(self.spaceList, 1, 2)

        #--3.1 some useful tool
        toolLayout = QHBoxLayout()
        ctrlLayout.addLayout(toolLayout)

        self.resetAllBttn = QPushButton()
        self.resetSelBttn = QPushButton()
        self.copyKeyBttn = QPushButton()
        self.pasteKeyBttn = QPushButton()
        self.keyAllBttn = QPushButton()
        self.deleteKeyBttn = QPushButton()
        self.cleanKeyBttn = QPushButton()

        [toolLayout.addWidget(bttn) for bttn in [self.resetAllBttn, self.resetSelBttn, self.copyKeyBttn, self.pasteKeyBttn, self.keyAllBttn, self.deleteKeyBttn, self.cleanKeyBttn]]

        #--4 version info
        self.versionTxt = QTextEdit()
        self.versionTxt.setReadOnly(True)
        self.vSplitter.addWidget(self.versionTxt)

        #--5
        self.editPickerBttn = QPushButton("Edit Picker")
        #self.vSplitter.addWidget(self.editPickerBttn)
        self.vSplitter.setSizes([20, 700, 150, 0, 0])

        #--6
        copyrightLabel = QLabel('2016 www.characterrigs.com. All rights reserved.')
        mainLayout.addWidget(copyrightLabel, alignment=Qt.AlignRight)

    def ctrlTabRightClick(self, point): 
        self.viewMenu.exec_(self.ctrlTree.mapToGlobal(point)) 

    def editPickerLayout(self):
        self.editPickerWidget = QWidget()

        #Parent widget under Maya main window
        self.editPickerWidget.setParent(self)
        self.editPickerWidget.setWindowFlags(Qt.Window)
        self.editPickerWidget.setObjectName("EditPickerUI")
        self.editPickerWidget.setWindowTitle("Edit Picker")
        self.editPickerWidget.setGeometry(580, 300, 300, 500)

        #--4 picker editor
        mainLayout = QVBoxLayout()
        self.editPickerWidget.setLayout(mainLayout)

        #--
        bttnLayout = QGridLayout()
        mainLayout.addLayout(bttnLayout)

        self.newTabBttn = QPushButton("New Tab")
        self.newPickerBttn = QPushButton("New Picker")
        self.copyPickerBttn = QPushButton("Copy Picker")
        self.mirrorPickerBttn = QPushButton("Mirror Picker")

        bttnLayout.addWidget(self.newTabBttn, 0, 0)
        bttnLayout.addWidget(self.newPickerBttn, 0, 1)
        bttnLayout.addWidget(self.copyPickerBttn, 1, 0)
        bttnLayout.addWidget(self.mirrorPickerBttn, 1, 1)

        #--
        settingLayout = QGridLayout()
        mainLayout.addLayout(settingLayout)

        nameLabel = QLabel("Name:")
        iconLabel = QLabel("Icon:")
        postionLabel = QLabel("Position:")
        ctrlsLabel = QLabel("Controls:")

        self.pickerNameLine = QLineEdit()
        self.iconList = QListWidget()
        self.posXSpin = QSpinBox()
        self.posYSpin = QSpinBox()
        posLayout = QHBoxLayout()
        self.posXSpin.setRange(0, 500)
        self.posYSpin.setRange(0, 500)
        ctrlLayout = QHBoxLayout()
        self.ctrlsLine = QLineEdit()
        self.editCtrlBttn = QPushButton("<<")

        settingLayout.addWidget(nameLabel, 0, 0)
        settingLayout.addWidget(iconLabel, 1, 0)
        settingLayout.addWidget(postionLabel, 2, 0)
        settingLayout.addWidget(ctrlsLabel, 3, 0)

        settingLayout.addWidget(self.pickerNameLine, 0, 1)
        settingLayout.addWidget(self.iconList, 1, 1)
        settingLayout.addLayout(posLayout, 2, 1)
        settingLayout.addLayout(ctrlLayout, 3, 1)

        ctrlLayout.addWidget(self.ctrlsLine)
        ctrlLayout.addWidget(self.editCtrlBttn)
        posLayout.addWidget(self.posXSpin)
        posLayout.addWidget(self.posYSpin)

        #---
        exportLayout1 = QGridLayout()
        mainLayout.addLayout(exportLayout1)
        pickerPathLabel = QLabel("Save Path:")
        pickerNameLable = QLabel("Save Name:")
        self.exportPathLine = QLineEdit()
        exportLayout1.addWidget(pickerPathLabel, 0, 0)
        exportLayout1.addWidget(self.exportPathLine, 0, 1)
        exportLayout1.addWidget(pickerNameLable, 1, 0)

        exportLayout2 = QHBoxLayout()
        exportLayout1.addLayout(exportLayout2, 1, 1)
        self.exportNameLine = QLineEdit()
        self.browsePathBttn = QPushButton("Browse")
        self.openPathBttn = QPushButton("Open")
        exportLayout2.addWidget(self.exportNameLine)
        exportLayout2.addWidget(self.browsePathBttn)
        exportLayout2.addWidget(self.openPathBttn)

        #--
        offsetLayout = QGridLayout()
        mainLayout.addLayout(offsetLayout)

        self.savePickerBttn = QPushButton("Save Picker")
        self.exportPickerBttn = QPushButton("Export Picker")
        self.importPickerBttn = QPushButton("Import Picker")

        self.movePickerUpBttn = QPushButton("^")
        self.movePickerLeftBttn = QPushButton("<")
        self.movePickerRightBttn = QPushButton(">")
        self.movePickerDownBttn = QPushButton("v")

        self.allAlignLeftBttn = QPushButton(">|")
        self.allAlignRightBttn = QPushButton("|<")
        self.allAlignUpBttn = QPushButton("^")
        self.allAlignLowBttn = QPushButton("v")

        offsetLayout.addWidget(self.savePickerBttn, 0, 0)
        offsetLayout.addWidget(self.exportPickerBttn, 1, 0)
        offsetLayout.addWidget(self.importPickerBttn, 2, 0)

        offsetLayout.addWidget(self.movePickerUpBttn, 0, 4)
        offsetLayout.addWidget(self.movePickerLeftBttn, 1, 3)
        offsetLayout.addWidget(self.movePickerRightBttn, 1, 5)
        offsetLayout.addWidget(self.movePickerDownBttn, 2, 4)
        #offsetLayout.addWidget(QLabel("     "), 0, 1)
        #offsetLayout.setColumnStretch(1, 1)
        offsetLayout.addWidget(self.allAlignLeftBttn, 0, 1)
        offsetLayout.addWidget(self.allAlignRightBttn, 0, 2)
        offsetLayout.addWidget(self.allAlignUpBttn, 1, 1)
        offsetLayout.addWidget(self.allAlignLowBttn, 1, 2)

        [bttn.setFixedWidth(100) for bttn in [self.savePickerBttn, self.exportPickerBttn, self.importPickerBttn]]
        [bttn.setFixedWidth(25) for bttn in [self.movePickerUpBttn, self.movePickerLeftBttn, self.movePickerRightBttn, self.movePickerDownBttn]]

        self.editCbx = QCheckBox("Edit Mode")
        mainLayout.addWidget(self.editCbx)

        self.printPickerBttn = QPushButton("Print Selected Picker")
        self.deleteTabBttn = QPushButton("Delete Current Tab")
        self.deletePickerBttn = QPushButton("Delete Selected Picker")
        mainLayout.addWidget(self.printPickerBttn)
        mainLayout.addWidget(self.deleteTabBttn)
        mainLayout.addWidget(self.deletePickerBttn)

        iconDir = os.path.join(self.iconDir, "Picker")
        if os.path.isdir(iconDir):
            for icon in sorted(os.listdir(iconDir)):
                if ".png" in icon:
                    iconItem = QListWidgetItem()
                    iconItem.setIcon(QIcon(os.path.join(iconDir, icon)))
                    iconItem.setText(icon.partition(".")[0])
                    self.iconList.addItem(iconItem)
                    iconItem.setSizeHint(QSize(100, 20))
        
    def newTabLayout(self):
        self.newTabWidget = QWidget()
        self.newTabWidget.setParent(self.editPickerWidget)
        self.newTabWidget.setWindowFlags(Qt.Window)
        self.newTabWidget.setObjectName("NewTabUI")
        self.newTabWidget.setWindowTitle("Create New Picker Tab")
        self.newTabWidget.setGeometry(900, 300, 300, 100)

        mainLayout = QVBoxLayout()
        nameLayout = QHBoxLayout()
        mainLayout.addLayout(nameLayout)
        self.newTabWidget.setLayout(mainLayout)

        nameLabel = QLabel("Name:")
        self.newTabLine = QLineEdit()
        self.createTabBttn = QPushButton("Create New Picker Tab")
        nameLayout.addWidget(nameLabel)
        nameLayout.addWidget(self.newTabLine)
        mainLayout.addWidget(self.createTabBttn)

    def newPickerLayout(self):
        self.newPickerWidget = QWidget()
        self.newPickerWidget.setParent(self.editPickerWidget)
        self.newPickerWidget.setWindowFlags(Qt.Window)
        self.newPickerWidget.setObjectName("NewPickerUI")
        self.newPickerWidget.setWindowTitle("Create New Picker")
        self.newPickerWidget.setGeometry(900, 300, 300, 300)

        #name icon postion ctrls
        mainLayout = QVBoxLayout()
        self.newPickerWidget.setLayout(mainLayout)
        settingLayout = QGridLayout()
        mainLayout.addLayout(settingLayout)

        nameLabel = QLabel("Name:")
        iconLabel = QLabel("Icon:")
        postionLabel = QLabel("Position:")
        ctrlsLabel = QLabel("Controls")

        self.newPickerNameLine = QLineEdit()
        self.newIconList = QListWidget()
        posLabyout = QHBoxLayout()
        self.newPosXSpin = QSpinBox()
        self.newPosYSpin = QSpinBox()
        self.newPosXSpin.setRange(0, 500)
        self.newPosYSpin.setRange(0, 500)
        ctrlLayout = QHBoxLayout()
        self.newCtrlsLine = QLineEdit()
        self.loadCtrlBttn = QPushButton("<<")

        settingLayout.addWidget(nameLabel, 0, 0)
        settingLayout.addWidget(iconLabel, 1, 0)
        settingLayout.addWidget(postionLabel, 2, 0)
        settingLayout.addWidget(ctrlsLabel, 3, 0)
        settingLayout.addWidget(self.newPickerNameLine, 0, 1)
        settingLayout.addWidget(self.newIconList, 1, 1)
        settingLayout.addLayout(posLabyout, 2, 1)
        settingLayout.addLayout(ctrlLayout, 3, 1)

        posLabyout.addWidget(self.newPosXSpin)
        posLabyout.addWidget(self.newPosYSpin)
        ctrlLayout.addWidget(self.newCtrlsLine)
        ctrlLayout.addWidget(self.loadCtrlBttn)

        self.createPickerBttn = QPushButton("Create New Picker")
        mainLayout.addWidget(self.createPickerBttn)

        iconDir = os.path.join(self.iconDir, "Picker")
        if os.path.isdir(iconDir):
            for icon in sorted(os.listdir(iconDir)):
                if ".png" in icon:
                    iconItem = QListWidgetItem()
                    iconItem.setIcon(QIcon(os.path.join(iconDir, icon)))
                    iconItem.setText(icon.partition(".")[0])
                    self.newIconList.addItem(iconItem)
                    iconItem.setSizeHint(QSize(100, 20))

    def mirrorPickerLayout(self):
        self.mirrorPickerWidget = QWidget()
        self.mirrorPickerWidget.setParent(self.editPickerWidget)
        self.mirrorPickerWidget.setWindowFlags(Qt.Window)
        self.mirrorPickerWidget.setObjectName("MirrorPickerUI")
        self.mirrorPickerWidget.setWindowTitle("Mirror Selected Pickers")
        self.mirrorPickerWidget.setGeometry(900, 300, 300, 100)

        mainLayout = QVBoxLayout()
        self.mirrorPickerWidget.setLayout(mainLayout)
        replaceLayout = QGridLayout()
        mainLayout.addLayout(replaceLayout)

        findLabel = QLabel("Find")
        replaceLabel = QLabel("Replace")
        nameLabel = QLabel("Name:")
        ctrlLabel = QLabel("Controls:")

        self.findNameLine = QLineEdit()
        self.replaceNameLine = QLineEdit()
        self.findCtrlLine = QLineEdit()
        self.replaceCtrlLine = QLineEdit()

        self.findNameLine.setText("lf") 
        self.replaceNameLine.setText("rt") 
        self.findCtrlLine.setText("LF_") 
        self.replaceCtrlLine.setText("RT_")

        replaceLayout.addWidget(findLabel, 0, 1)
        replaceLayout.addWidget(replaceLabel, 0, 2)
        replaceLayout.addWidget(nameLabel, 1, 0)
        replaceLayout.addWidget(ctrlLabel, 2, 0)

        replaceLayout.addWidget(self.findNameLine, 1, 1)
        replaceLayout.addWidget(self.replaceNameLine, 1, 2)
        replaceLayout.addWidget(self.findCtrlLine, 2, 1)
        replaceLayout.addWidget(self.replaceCtrlLine, 2, 2)

        self.doMirrorPickerBttn = QPushButton("Mirror Selected Pickers")
        mainLayout.addWidget(self.doMirrorPickerBttn)

    def customizeWidget(self):
        '''
        set icon to some widget
        '''
        self.buttonStyle = "QPushButton{background: transparent; border: 0px} QPushButton:pressed {background-color: rgb(100, 100, 100);}"

        buttnDic = {self.refreshBttn: ["refresh.png", "Refresh"],
                    self.showVisBttn: ["eye_open.png", "Show All Controls"],
                    self.hideVisBttn: ["eye_close.png", "Hide All Controls"],
                    self.isolateVisBttn: ["eye_reign.png", "Isolate Selected Controls"],

                    self.resetAllBttn: ["tpose.png", 'Reset All'],
                    self.resetSelBttn: ["tpose_reign.png", 'Reset Sel'],
                    self.copyKeyBttn: ["copy.png", 'Copy Attr'],
                    self.pasteKeyBttn: ["stamp.png", 'Paste Attr'],
                    self.keyAllBttn: ["key.png", 'key All'],
                    self.deleteKeyBttn: ["delete.png", 'key All'],
                    self.cleanKeyBttn: ["clean.png", 'Delete Key']}

        for bttn, item in buttnDic.items():
            bttn.setStyleSheet(self.buttonStyle)
            bttn.setToolTip(item[1])
            iconPath = os.path.join(self.iconDir, item[0])
            if os.path.isfile(iconPath):
                bttn.setIcon(QIcon(iconPath))
            else:
                bttn.setText(item[1])
        return True

    def changeUIOpacity(self):
        v = self.alphaSlider.value()
        self.setWindowOpacity(v/100.0)

    #----------------------------------------------------------------------------------------------
    def modeSwitch(self):
        try:
            mode = cmds.evaluationManager(mode=True, q=True)

            if mode[0] == "parallel":
                cmds.evaluationManager(mode="off")
                cmds.headsUpMessage("Switch mode to DG")
                self.modeBttn.setText("   DG    ")
            elif mode[0] == "off": 
                cmds.evaluationManager(mode="parallel")
                cmds.headsUpMessage("Switch mode to parallel")
                self.modeBttn.setText("Parallel")
        except:
            return False

    #----------------------------------------------------------------------------------------------  
    def loadCharacters(self):
        self.characterCombo.clear()
        if not self.editCbx.isChecked():
            #get name space from maya scene
            self.charatcers = []
            references = cmds.file(r=True, q=True)
            for ref in references:
                if os.path.isfile(ref):
                    refNode = cmds.file(ref, q=True, rfn=True)
                    nameSpace = cmds.file(ref, q=True, ns=True)
                    nameSpace2 = cmds.referenceQuery(refNode, ns=True)
                    load = cmds.referenceQuery(refNode, il=True)                
                    sep = "_" if nameSpace2 == ":" else ":"
                    if cmds.objExists('{0}{1}World'.format(nameSpace, sep)):
                        self.charatcers.append(nameSpace)
            self.characterCombo.addItems(self.charatcers)
        else:
            self.characters = []

        self.getVersionInfo()
        self.updateCtrlTree()
        self.updateIKFKList()
        self.updateSpaceCtrlList()
        self.updateCtrlPicker()

    #------------------------------------------version info--------------------------------------------------
    def getVersionInfo(self):
        self.versionTxt.clear()

        if not self.editCbx.isChecked():
            self.character = self.characterCombo.currentText()
            if not self.character:
                print "Cant't find any reference character..."
                return False
        else:
            self.character = ""

        print "Set current character: {0}".format(self.character)

        try:
            versionInfo = cmds.getAttr('{0}:Credit_Curve.notes'.format(self.character))
            QversionInfo = QTextDocument(versionInfo)
            self.versionTxt.setDocument(QversionInfo)
        except:
            return False

    #-------------------------------------------build tree---------------------------------------------------         
    def updateCtrlTree(self):
        self.treeModel.clear()

        if not self.editCbx.isChecked():
            if not self.character:
                self.treeModel.appendRow(QStandardItem('Please Use Rig as Reference'))
                self.treeModel.appendRow(QStandardItem('How:'))
                self.treeModel.appendRow(QStandardItem('    ->File'))
                self.treeModel.appendRow(QStandardItem('    ->Create Reference...'))
                self.treeModel.appendRow(QStandardItem('    ->Select the Rig File'))
                self.treeModel.appendRow(QStandardItem('    ->Click Refresh'))
                return False

        parentItem = '{0}:Ctrls_Set'.format(self.character)
        if cmds.objExists(parentItem):
            QparentItem = QStandardItem('Ctrls_Set')
            self.treeModel.appendRow(QparentItem)
            self.pupulateCtrlTree(parentItem, QparentItem)

            # expand the container
            index = self.treeModel.indexFromItem(QparentItem)
            self.ctrlTree.expand(index)
        else:
            QparentItem = QStandardItem(self.character + ':Ctrls_Set is missing')
            self.treeModel.appendRow(QparentItem)

    def pupulateCtrlTree(self, parentItem, QparentItem):
        color = {'LF_Ctrl': QColor(170, 100, 100), 'MD_Ctrl': QColor(100, 170, 100), 'RT_Ctrl': QColor(100, 170, 170),
                 'LF_Set': QColor(240, 80, 80), 'MD_Set': QColor(80, 240, 80), 'RT_Set': QColor(80, 240, 240)}

        if cmds.objectType(parentItem, i='objectSet'):
            subItems = []
            subSetItems = cmds.listConnections(parentItem+'.dnSetMembers', d=False)
            subCtrlItems = cmds.listConnections(parentItem+'.dagSetMembers', d=False)
            if subSetItems:
                subItems.extend(subSetItems)
            if subCtrlItems:
                subItems.extend(subCtrlItems)

            if len(subItems) > 0:    
                for subItem in sorted(subItems):
                    ctrl = subItem.rpartition(":")[2]
                    QsubItem = QStandardItem(ctrl)
                    QparentItem.appendRow(QsubItem)

                    side = ctrl.partition('_')[0]
                    objType = ctrl.rpartition('_')[2]
                    if side not in ['RT', 'LF']: side ='MD'
                    colorIndex = '{0}_{1}'.format(side, objType)

                    QsubItem.setForeground(QBrush(color[colorIndex]))
                    self.pupulateCtrlTree(subItem, QsubItem)
        else:
            return 1
   
    def selectTreeCtrls(self):
        ctrls = ["{0}:{1}".format(self.character, self.treeModel.itemFromIndex(Qindex).text()) for Qindex in self.ctrlTree.selectedIndexes()]
        try:
            cmds.select(ctrls, r=True)
        except:
            return False

        if ctrls[0] in self.spaceCtrls:
            self.spaceCtrlList.setCurrentItem(self.spaceCtrlList.findItems(ctrls[0].partition(":")[2], Qt.MatchExactly)[0])

        if self.isoSelectAction.isChecked():
            self.showHideAllCtrls(0)
            for ctrl in ctrls:
                ctrlShapes = cmds.listRelatives(ctrl, s=True)
                for ctrlShape in ctrlShapes:
                    if cmds.getAttr('{0}.v'.format(ctrlShape), se=True):
                        cmds.setAttr('{0}.v'.format(ctrlShape), 1)
 
    def showHideAllCtrls(self, vis):
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        ctrls = cmds.ls('{0}:*_Ctrl'.format(self.character))

        for ctrl in ctrls:
            ctrlShapes = cmds.listRelatives(ctrl, s=True)
            for ctrlShape in ctrlShapes:
                if cmds.getAttr('{0}.v'.format(ctrlShape), se=True):
                    cmds.setAttr('{0}.v'.format(ctrlShape), vis)

        for grp in ["MD_Throat_Ctrl_BK_Grp", "MD_Breathe_Ctrl_BK_Grp", "LF_Leg_01_PV_Anno", "RT_Leg_01_PV_Anno", "LF_Arm_01_PV_Anno", "RT_Arm_01_PV_Anno"]:
            if cmds.objExists("{0}:{1}".format(self.character, grp)):
                cmds.setAttr("{0}:{1}.v".format(self.character, grp), vis)

    def isolateSelectedCtrls(self):
        '''
        make only selected control visibile
        '''
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        ctrls = [s for s in cmds.ls(sl=True) if "_Ctrl" in s]
        if ctrls:
            self.showHideAllCtrls(0)
            try:
                for ctrl in ctrls:
                    ctrlShapes = cmds.listRelatives(ctrl, s=True)
                    for ctrlShape in ctrlShapes:
                        if cmds.getAttr('{0}.v'.format(ctrlShape), se=True):
                            cmds.setAttr('{0}.v'.format(ctrlShape), 1)
            except:
                return False

    #-------------------------------------------ik fk switch---------------------------------------------------
    def updateIKFKList(self):
        color = {'LF': QColor(240, 120, 120), 'RT': QColor(120, 240, 240)}
        self.ikfkList.clear()

        if not self.editCbx.isChecked():
            if not self.character:
                return False

        ikfkSwitchCtrls = cmds.ls('{0}:*IKFK_Switch_Ctrl'.format(self.character))
        if ikfkSwitchCtrls:
            for ikfkSwitchCtrl in ikfkSwitchCtrls:
                limb = ikfkSwitchCtrl.rpartition(":")[2].rsplit('_', 3)[0]
                side = limb.partition('_')[0]
                if cmds.getAttr("{0}.IKFKSwitch".format(ikfkSwitchCtrl)) == 1:
                    IKFKItem = QListWidgetItem("{0}:-->IK".format(limb))
                else:
                    IKFKItem = QListWidgetItem("{0}:-->FK".format(limb))
                iconPath = os.path.join(self.iconDir, "switch.png")
                if os.path.isfile(iconPath):
                    IKFKItem.setIcon((QIcon(iconPath)))
                if side in ['LF', 'RT']:
                    IKFKItem.setForeground(QBrush(color[side]))
                else:
                    IKFKItem.setForeground(QBrush(QColor(240, 240, 120)))  
                self.ikfkList.addItem(IKFKItem)

    def doikfkSwitch(self):
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        item = self.ikfkList.currentItem()
        if item:
            limb = item.text().partition(":")[0]
            self.ikfkSwitch(namespace=self.character+":", longAlias=limb, item=item)

    def ikfkSwitch(self, namespace="", longAlias="LF_Arm_01", item=None):
        '''
        do ik fk switch
        '''
        side = longAlias.split("_")[0]
        alias =longAlias.split("_")[1]
        index = int(longAlias.split("_")[2])

        left = "LF"
        right = "RT"
        aliasListDic = {"Arm": ["Shoulder", "Elbow", "Wrist"], "Leg": ["Hip", "Knee", "Ankle", "Foot", "Toe"],
                        "ForeLeg": ["Hip", "Knee", "Ankle", "Foot", "Knuckle", "Toe"], 
                        "HindLeg": ["Hip", "Knee", "Ankle", "Foot", "Knuckle", "Toe"]}
        aliasList = aliasListDic[alias]

        ikCtrl = "{0}_IK_Ctrl".format(longAlias)
        pvCtrl = "{0}_PV_Ctrl".format(longAlias)

        upFkCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[0])
        midFkCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[1])
        lowFkCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[2])

        upJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[0])
        midJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[1])
        lowJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[2])

        if alias in ["Leg", "ForeLeg", "HindLeg"]:
            footJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[3])
            toeJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[-1])

            footIkCtrl = "{0}_{1}_IK_Ctrl".format(longAlias, aliasList[3])
            footFkCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[3])

            toeIkCtrl = "{0}_{1}_IK_Ctrl".format(longAlias, aliasList[-1])
            toeFkCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[-1])

            if alias in ["ForeLeg", "HindLeg"]:
                kunckJnt = "{0}_{1}_Jnt".format(longAlias, aliasList[4])
                kunckIKCtrl = "{0}_{1}_IK_Ctrl".format(longAlias, aliasList[4])
                kunckFKCtrl = "{0}_{1}_FK_Ctrl".format(longAlias, aliasList[4])

                toeSubIKCtrl = "{0}_{1}_IK_Sub_Ctrl".format(longAlias, aliasList[-1])

        ikfkCtrlAttr = '{0}_IKFK_Switch_Ctrl.IKFKSwitch'.format(longAlias)
        ikValue = 1
        fkValue = 0

        pvAxisDic = {"Arm": "-z", "Leg": "z", "ForeLeg": "-z", "HindLeg": "z"}
        pvAxis = pvAxisDic[alias]
        '''
        in my setup right side fk ctrls have (180, 0, 0)
        ik ctrls' offset is stored in their UD attributes
        '''

        if cmds.getAttr(namespace+ikfkCtrlAttr) == ikValue:
            print 'do switch from ik to fk for {0} :)'.format(longAlias)
            item.setText("{0}:-->FK".format(longAlias))

            jnts = [upJnt, midJnt, lowJnt]
            fkCtrls = [upFkCtrl, midFkCtrl, lowFkCtrl]
            if alias in ["Leg"]:
                jnts.extend([footJnt, toeJnt])
                fkCtrls.extend([footFkCtrl, toeFkCtrl])

            if alias in ["ForeLeg", "HindLeg"]:
                jnts.extend([footJnt, kunckJnt, toeJnt])
                fkCtrls.extend([footFkCtrl, kunckFKCtrl, toeFkCtrl])

            poss = []
            oris = []
            for jnt in jnts:
                pos = cmds.xform(namespace+jnt, q=True, t=True, ws=True)
                ori = cmds.xform(namespace+jnt, q=True, ro=True, ws=True)
                poss.append(pos)
                oris.append(ori)

            cmds.setAttr(namespace+ikfkCtrlAttr, fkValue)

            for i, ctrl in enumerate(fkCtrls):
                if cmds.objExists(namespace+ctrl):
                    cmds.xform(namespace+ctrl, t=poss[i], ro=oris[i], ws=True, a=True)
                    if side == right:
                        cmds.xform(namespace+ctrl, ro=(180, 0, 0), os=True, r=True)

        elif cmds.getAttr(namespace+ikfkCtrlAttr) == fkValue:
            print 'do switch from fk to ik for {0} :)'.format(longAlias)
            item.setText("{0}:-->IK".format(longAlias))

            ikPos = cmds.xform(namespace+footJnt, q=True, t=True, ws=True) if alias in ["HindLeg"] else cmds.xform(namespace+lowJnt, q=True, t=True, ws=True) 
            ikOri = cmds.xform(namespace+footJnt, q=True, ro=True, ws=True) if alias in ["HindLeg"] else cmds.xform(namespace+lowJnt, q=True, ro=True, ws=True) 
            ikOffset = cmds.getAttr('{0}.ikOffset'.format(namespace+ikCtrl))[0]
            pvPos = self.findPV(namespace+upJnt, namespace+midJnt, namespace+lowJnt, pvAxis)


            if alias in ["Leg", "ForeLeg", "HindLeg"]:
                footMX = cmds.xform(namespace+footJnt, q=True, m=True, ws=True)
                upVec = footMX[4:7]
                if side == left:
                    upVec = self.vMultiply(upVec, -1)
                footPos = cmds.xform(namespace+footJnt, q=True, t=True, ws=True)
                toePos = cmds.xform(namespace+toeJnt, q=True, t=True, ws=True)
                aimVec = self.vPlus(footPos, toePos, "-")
                toeOri = self.vectorToAngle(aimVec, upVec=upVec, sideVec=None, aimAxis="z", upAxis="x", sideAxis=None)

            #get foot ik's orientation
            if alias in ["HindLeg"]:
                lowMX = cmds.xform(namespace+lowJnt, q=True, m=True, ws=True)
                upVec = lowMX[4:7]
                if side == left:
                    upVec = self.vMultiply(upVec, -1)
                lowPos = cmds.xform(namespace+lowJnt, q=True, t=True, ws=True)
                aimVec = self.vPlus(lowPos, footPos, "-")
                footOri = self.vectorToAngle(aimVec, upVec=upVec, sideVec=None, aimAxis="z", upAxis="x", sideAxis=None)

            cmds.setAttr(namespace+ikfkCtrlAttr, ikValue)

            cmds.xform(namespace+ikCtrl, t=ikPos, ro=ikOri, ws=True, a=True)
            cmds.xform(namespace+ikCtrl, ro=ikOffset, os=True, r=True)
            cmds.xform(namespace+pvCtrl, t=pvPos, ws=True, a=True)

            if alias in ["Leg", "ForeLeg", "HindLeg"]:
                cmds.xform(namespace+toeIkCtrl, ro=toeOri, a=True, ws=True)

                if alias in ["HindLeg"]:
                    cmds.xform(namespace+footIkCtrl, ro=footOri, a=True, ws=True)
                else:
                    cmds.xform(namespace+footIkCtrl, ro=(0, 0, 0), a=True, os=True)

            if alias in ["ForeLeg", "HindLeg"]:
                if cmds.objExists(namespace+kunckIKCtrl):
                    cmds.setAttr("{0}.r".format(namespace+kunckIKCtrl), 0, 0, 0, type="double3")

                if cmds.objExists(namespace+toeSubIKCtrl):
                    cmds.setAttr("{0}.r".format(namespace+toeSubIKCtrl), 0, 0, 0, type="double3")

        return True

    def findPV(self, obj1, obj2, obj3, axis, scale=10):
        neg = 1
        if axis[0] == '-':
            neg = -1
            axis = axis[1]

        row = {'x': 0, 'y': 1, 'z': 2}

        P = [cmds.xform(obj, q=True, t=True, ws=True) for obj in [obj1, obj2, obj3]]

        V0 = self.vPlus(P[2], P[0], '-')
        V1 = self.vPlus(P[1], P[0], '-')
        w = self.vDot(V0, V1) / self.vDot(V0, V0)
        Vp = self.vMultiply(V0, w)
        Pp = self.vPlus(P[0], Vp, '+') 

        if 1 - self.vDot(V0, V1)/(self.vLength(V0)*self.vLength(V1)) < 0.0001:
            M16 = cmds.xform(obj1, q=True, matrix=True, ws=True)
            M3x3 = []
            for i in range(3):
                M3x3.append([M16[i*4], M16[i*4+1], M16[i*4+2]])
            Vperp = self.vMultiply(M3x3[row[axis]], neg)
        else:
            Vperp = self.vPlus(P[1], Pp, '-')

        Vperp = self.vNormal(Vperp)
        Vperp = self.vMultiply(Vperp, scale)
        V0 = self.vMultiply(V0, 0.5)
        Pmid = self.vPlus(P[0], V0, '+')
        pvPos = self.vPlus(Pmid, Vperp, '+')

        return pvPos

    #-------------------------------------------space switch---------------------------------------------------
    def updateSpaceCtrlList(self):
        color = {'LF': QColor(240, 120, 120), 'RT': QColor(120, 240, 240)}

        self.spaceCtrlList.clear()
        self.spaceCtrls = []

        if not self.editCbx.isChecked():
            if not self.character:
                return False

        allCtrls = cmds.ls('{0}:*_Ctrl'.format(self.character))
        self.spaceCtrls = [ctrl for ctrl in allCtrls if cmds.objExists('{0}.space'.format(ctrl)) or cmds.objExists('{0}.orientSpace'.format(ctrl))]
        
        if self.spaceCtrls:
            for spaceCtrl in sorted(self.spaceCtrls):
                ctrl = spaceCtrl.rpartition(":")[2]
                side = ctrl.partition('_')[0]
                ctrlItem = QListWidgetItem(ctrl)
                self.spaceCtrlList.addItem(ctrlItem)
                if side in ['LF', 'RT']:
                    ctrlItem.setForeground(QBrush(color[side]))
                else:
                    ctrlItem.setForeground(QBrush(QColor(240, 240, 120)))                  

    def updateSpaceList(self):
        self.spaceList.clear()
        ctrlItem = self.spaceCtrlList.currentItem()
        if ctrlItem:
            spaceCtrl = "{0}:{1}".format(self.character, ctrlItem.text())

            if cmds.objExists(spaceCtrl):
                try:
                    strSpaceValues = cmds.attributeQuery('space', node=spaceCtrl, le=True)
                    attr = 'space'
                except:
                    strSpaceValues = cmds.attributeQuery('orientSpace', node=spaceCtrl, le=True)
                    attr = 'orientSpace'

                if strSpaceValues:
                    spaceValues = strSpaceValues[0].split(':')
                    for spaceValue in spaceValues:
                        spaceItem = QListWidgetItem(spaceValue)
                        spaceItem.setForeground(QBrush(QColor(230, 230, 150)))                  
                        self.spaceList.addItem(spaceItem)

                    row = cmds.getAttr('{0}.{1}'.format(spaceCtrl, attr))
                    self.spaceList.setCurrentRow(row)

            #select Space Ctrl
            cmds.select(spaceCtrl, r=True)

    def spaceSwitch(self):
        ctrlItem = self.spaceCtrlList.currentItem()
        if ctrlItem:
            spaceCtrl = "{0}:{1}".format(self.character, ctrlItem.text())
            pos = cmds.xform(spaceCtrl, q=True, t=True, ws=True)
            ori = cmds.xform(spaceCtrl, q=True, ro=True, ws=True)

            spaceIndex = self.spaceList.currentRow()
            try:
                if cmds.objExists('{0}.space'.format(spaceCtrl)):
                    cmds.setAttr('{0}.space'.format(spaceCtrl), spaceIndex)
                else:
                    cmds.setAttr('{0}.orientSpace'.format(spaceCtrl), spaceIndex)
                cmds.xform(spaceCtrl, t=pos, ro=ori, ws=True)
            except:
                return False

    #--------------------------------------------some useful tool----------------------------------------------
    def keyAllCtrls(self):
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        ctrls = cmds.ls('{0}:*_Ctrl'.format(self.character))
        if ctrls:
            cmds.setKeyframe(ctrls)
            cmds.select(ctrls)

    def resetAllCtrls(self):
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        ctrls = cmds.ls('{0}:*_Ctrl'.format(self.character))
        if ctrls:
            self.resetCtrls(ctrls)

    def resetSelectedCtrls(self):
        ctrls = [s for s in cmds.ls(sl=True) if "_Ctrl" in s]
        if ctrls:
            self.resetCtrls(ctrls)

    def resetCtrls(self, ctrls, ud=True):
        [cmds.xform(ctrl, t=(0, 0, 0), ro=(0, 0, 0), s=(1, 1, 1), a=True, os=True) for ctrl in ctrls]

        if ud:
            for ctrl in ctrls:        
                UDAttrs = cmds.listAttr(ctrl, ud=True)
                if UDAttrs:
                    for UDAttr in UDAttrs:               
                        if cmds.getAttr('{0}.{1}'.format(ctrl, UDAttr), se=True):                                             
                            ValueType = cmds.addAttr('{0}.{1}'.format(ctrl, UDAttr), q=True, dt=True)[0]                       
                            if ValueType != 'string':
                                defaultValue = cmds.addAttr('{0}.{1}'.format(ctrl, UDAttr), q=True, dv=True)
                                cmds.setAttr('{0}.{1}'.format(ctrl, UDAttr), defaultValue)

    def copyAttr(self):
        #clear data container first
        self.tmpAttr = {}
        ctrl = cmds.ls(sl=True)
        if len(ctrl) == 1:
            if ctrl[0].rpartition('_')[2] == 'Ctrl':
                for attr in ['tx', 'ty', 'tz', 'rx', 'ry', 'rz', 'sx', 'sy', 'sz']:
                    if cmds.getAttr(ctrl[0] + '.' + attr, se=True):
                        self.tmpAttr[attr] = cmds.getAttr(ctrl[0] + '.' + attr)

                UDAttrs = cmds.listAttr(ctrl[0], ud=True)
                if UDAttrs:
                    for UDAttr in UDAttrs:                 
                        if cmds.getAttr(ctrl[0] + '.' + UDAttr, se=True):                                             
                            ValueType = cmds.addAttr(ctrl[0] + '.' + UDAttr, q=True, dt=True)[0]                       
                            if ValueType != 'string':
                                self.tmpAttr[UDAttr] = cmds.getAttr(ctrl[0] + '.' + UDAttr)

                print self.tmpAttr

    def pasteAttr(self):
        if self.tmpAttr:
            ctrls = cmds.ls(sl=True)
            for ctrl in ctrls:
                if ctrl.rpartition('_')[2] == 'Ctrl':
                    attrs = self.tmpAttr.keys()
                    for attr in attrs:
                        if cmds.objExists(ctrl + '.' + attr):
                            if cmds.getAttr(ctrl + '.' + attr, se=True):
                                cmds.setAttr(ctrl + '.' + attr, self.tmpAttr[attr])

    def deleteKey(self):
        ctrls = cmds.ls(sl=True)

        if ctrls:
            try:
                mel.eval('timeSliderClearKey')
            except:
                return False

    #----------------------------------------------some vector caculation------------------------------------------------
    #--------------------------------------------------------------------------------------------------------------------
    def vPlus(self, vec1, vec2, operator):
        '''operator: +, -'''
        vec = []      
        for i in range(len(vec1)):
            if operator == '+':
                vec.append(vec1[i]+vec2[i])
            elif operator == '-':
                vec.append(vec1[i]-vec2[i])
        return vec

    def vCross(self, vec1, vec2):
        vec = [vec1[1]*vec2[2] - vec1[2]*vec2[1],
               vec1[2]*vec2[0] - vec1[0]*vec2[2],
               vec1[0]*vec2[1] - vec1[1]*vec2[0]]
        return vec

    def vDot(self, vec1, vec2):
        return sum([vec1[i]*vec2[i] for i in range(len(vec1))])

    def vLength(self, vec1):
        return math.sqrt(self.vDot(vec1, vec1))

    def vMultiply(self, vec1, m):
        vec = []
        for i in range(len(vec1)):
            vec.append(vec1[i] * m)
        return vec

    def vNormal(self, vec1):
        vec1L = self.vLength(vec1)
        nVec1 = self.vMultiply(vec1, 1.0/vec1L)
        return nVec1

    def vSum(self, vec1):
        return sum([vec1[i] for i in range(len(vec1))])

    def vSquare(self, vec1):
        vec = []
        for i in range(len(vec1)):
            vec.append(vec1[i] * vec1[i])
        return vec

    def vectorToAngle(self, aimVec, upVec=None, sideVec=None, aimAxis="x", upAxis=None, sideAxis=None):
        '''
        convert 2 vector to a rotation value, the first vector is the aim vector, the second vector is the up vector
        input:
            aimVec(double3): the vector will absolutely matching one of the joint's defined axis
            upVec(double3): if using upVec, then don't use sideVec
            sideVec(double3): if using sideVec, then don't use upVec
            aimAxis(string): valid value: x, y, z
            upAxis(string): valid value: x, y, z
            sideAxis(string): valid value: x, y, z
        '''
        axisDic = {"x": 0, "y": 1, "z": 2}

        #if both side vector and up vector are defined, then side vector will be None 
        if upVec and sideVec:
            sideVec = None
        elif not upVec and not sideVec:
            return False

        if upAxis and sideAxis:
            sideAxis = None
        elif not upAxis and not sideAxis:
            return False

        if not sideAxis:
            sideAxis = axisDic.keys()
            sideAxis.remove(aimAxis)
            sideAxis.remove(upAxis)
            sideAxis = sideAxis[0]
        if not upAxis:
            upAxis = axisDic.keys()
            upAxis.remove(aimAxis)
            upAxis.remove(sideAxis)
            upAxis = upAxis[0]

        if not sideVec:
            sideVec = self.vCross(aimVec, upVec)
            upVec = self.vCross(aimVec, sideVec)
        if not upVec:
            upVec = self.vCross(aimVec, sideVec)
            sideVec = self.vCross(aimVec, upVec)

        #using a list to represent a 4 * 4 matrix
        M16 = [0, 0, 0, 1, #axisX
               0, 0, 0, 1, #axisY
               0, 0, 0, 1, #axisZ
               0, 0, 0, 1] #position

        #assign vector
        for i in range(3):
            M16[axisDic[aimAxis]*4+i] = aimVec[i]
            M16[axisDic[upAxis]*4+i] = upVec[i]
            M16[axisDic[sideAxis]*4+i] = sideVec[i]

        #convert matrix format    
        m = om.MMatrix(M16)              
        mTransformMtx = om.MTransformationMatrix(m)

        #decompose matrix to get a rotate value
        eulerRot = mTransformMtx.rotation().reorder(om.MEulerRotation.kXYZ)
        angle = [math.degrees(angle) for angle in (eulerRot.x, eulerRot.y, eulerRot.z)]

        return angle

    #------------------------
    def cleanAttrAnimationCurve(self, attribute, tols):
        #tols means tolerance
        values = cmds.keyframe(attribute, q = True, vc = True)
        times = cmds.keyframe(attribute, q = True, tc = True)
        inTnagent = cmds.keyTangent(attribute, q=True, inAngle=True)
        outTnagent = cmds.keyTangent(attribute, q=True, outAngle=True)
        
        #get a secondary value
        secValues = []  
        for i in range(len(values)-2):
            secValue = cmds.getAttr(attribute, t = times[i+1]+0.01)
            secValues.append(secValue)
        
        #a useless key is a key which after being deleted, does no change to the original animation curve
        for i in range(len(values)-2):
            cmds.cutKey(attribute, time=(times[i+1],times[i+1]))
            newValue = cmds.getAttr(attribute, t = times[i+1]+0.01)
            if abs(newValue - secValues[i])> tols:
                cmds.setKeyframe(attribute, t = times[i+1], v = values[i+1])
                cmds.keyTangent(attribute, edit=True, time=(times[i+1],times[i+1]), absolute=True, outAngle=outTnagent[i+1], ia=inTnagent[i+1])
                
    def cleanObjAnimationCurve(self, tols=0.00001):
        #clean object's animation curve

        objects = cmds.ls(sl = True)
        if len(objects) > 0:
            for obj in objects:
                allAttrs = cmds.listAttr(obj, k=True)
            
                keyedAttrs = []
                for objAttr in allAttrs:
                    keyNO = cmds.keyframe("{0}.{1}".format(obj, objAttr), q=True, kc=True)
                    if keyNO > 0:
                        keyedAttrs.append(objAttr)
                        
                for keyedAttr in keyedAttrs:
                    self.cleanAttrAnimationCurve("{0}.{1}".format(obj, keyedAttr), tols)

    #-------------------------------------------------draw picker---------------------------------------------
    #---------------------------------------------------------------------------------------------------------
    def getPickerData(self):
        self.pickerDic = {}
        if not self.editCbx.isChecked():
            if not self.character:
                return False

        #get data from the scene
        pickerAttr = "{0}:World.pickerDic".format(self.character)
        if cmds.objExists(pickerAttr):
            strPickerDic = cmds.getAttr(pickerAttr)
            self.pickerDic =ast.literal_eval(strPickerDic)

    def updateCtrlPicker(self):
        self.getPickerData()
        self.drawPickerLayout()

    def drawPickerLayout(self):
        #cleaning previous tab
        for i in range(self.ctrlTab.count()-1):
            graphicsView = self.ctrlTab.widget(1)
            graphicScene = graphicsView.scene()
            graphicScene.clear()
            self.ctrlTab.removeTab(1)
            graphicsView.deleteLater()

        #draw in new tab
        self.ctrlPickerDic = {} #a dictionary mapping from pixmap to controls
        self.bttnDic = {} # a dictional mapping from pixmap to bttnname

        for tab in sorted(self.pickerDic.keys()):
            graphicScene = QGraphicsScene()
            graphicsView = QGraphicsView(graphicScene)
            self.ctrlTab.addTab(graphicsView, tab)

            if not os.path.isdir(self.iconDir):
                graphicScene.addText("Warning! Please copy the Icons folder to the script directory!")

            graphicsView.setInteractive(True)
            graphicsView.setDragMode(QGraphicsView.RubberBandDrag)
            graphicsView.setSceneRect(0, 0, 450, 510)

            #connect with selection
            graphicScene.selectionChanged.connect(partial(self.selectPickerCtrls, graphicScene, tab))

            for bttnName in self.pickerDic[tab].keys():
                iconLabel = self.pickerDic[tab][bttnName][0]
                x = self.pickerDic[tab][bttnName][1][0]
                y = self.pickerDic[tab][bttnName][1][1]
                ctrls = self.pickerDic[tab][bttnName][2]
                iconPath = os.path.join(self.iconDir, "Picker", "{0}.png".format(iconLabel))

                #clean ctrl list
                [ctrls.remove(ctrl) for ctrl in ctrls if not ctrl]
                self.pickerDic[tab][bttnName][2] = ctrls

                if os.path.isfile(iconPath):
                    bttnItem = graphicScene.addPixmap(iconPath)
                    if self.editCbx.isChecked():
                        bttnItem.setFlag(QGraphicsItem.ItemIsMovable)
                    bttnItem.setFlag(QGraphicsItem.ItemIsSelectable)
                    bttnItem.moveBy(x, y)
                    #bttnItem.setScale(0.8)
                    self.ctrlPickerDic[bttnItem] = ctrls
                    self.bttnDic[bttnItem] = bttnName

            txtItem = graphicScene.addText("Hold ctrl key for multiple selection")
            txtItem.moveBy(-10, 520)
            txtItem.setDefaultTextColor(QColor(Qt.darkGray))

        self.ctrlTab.setCurrentIndex(1)
        self.getCurrentTab()
        return True

    def selectPickerCtrls(self, graphicScene, tab, *arg):
        ss = graphicScene.selectedItems()

        ctrls = []
        [[ctrls.append("{0}:{1}".format(self.character, ctrl)) for ctrl in self.ctrlPickerDic[s] if cmds.objExists("{0}:{1}".format(self.character, ctrl))] for s in ss]

        #print ctrls
        if self.editCbx.isChecked():
            self.showIconInfo(graphicScene, tab)
        try:
            cmds.select(ctrls, r=True)
            #self.createSelectionScriptJob(graphicScene)

            if ctrls[0] in self.spaceCtrls:
                self.spaceCtrlList.setCurrentItem(self.spaceCtrlList.findItems(ctrls[0].partition(":")[2], Qt.MatchExactly)[0])

            if self.isoSelectAction.isChecked():
                self.showHideAllCtrls(0)
                for ctrl in ctrls:
                    ctrlShapes = cmds.listRelatives(ctrl, s=True)
                    for ctrlShape in ctrlShapes:
                        if cmds.getAttr('{0}.v'.format(ctrlShape), se=True):
                            cmds.setAttr('{0}.v'.format(ctrlShape), 1)
        except:
            return False

    #---
    '''
    def createSelectionScriptJob(self, graphicScene):
        cScene = graphicScene.objectName()
        tmpLayout = cmds.lsUI(cl=True)[0]
        scriptJobNo = cmds.scriptJob(event=["SelectionChanged", partial(self.deselectionBttn, graphicScene)], runOnce=True, p=tmpLayout)

    def deselectionBttn(self, graphicScene):
        graphicScene.clearSelection()

        print "ss"
        bttnList = list(self.ctrlPickerDic.keys())
        ctrlList = list(self.ctrlPickerDic.values())

        ss = cmds.ls(sl=True)
        for ctrl in ss:
            if [ctrl.partition(":")[2]] in ctrlList:
                bttn = bttnList[ctrlList.index([ctrl.partition(":")[2]])]
                bttn.setSelected(True)
        #self.createSelectionScriptJob(graphicScene)
    '''

    #------------------------------------------------------------picker designer-------------------------------------------------------------
    #----------------------------------------------------------------------------------------------------------------------------------------
    def showIconInfo(self, graphicScene, tab):
        #self.pickerNameLine.clear()
        #self.ctrlsLine.clear()

        ss = graphicScene.selectedItems()
        self.currentItem = None

        if ss:
            self.posXSpin.setValue(ss[0].pos().x())
            self.posYSpin.setValue(ss[0].pos().y())
            self.currentItem = ss[0]
            bttnName = self.bttnDic[ss[0]]
            icon = self.pickerDic[tab][bttnName][0]
            ctrls = self.pickerDic[tab][bttnName][2]
            strCtrls = ""
            for ctrl in ctrls:
                strCtrls += ctrl + " "

            self.pickerNameLine.setText(bttnName)
            self.ctrlsLine.setText(strCtrls)
            self.iconList.setCurrentItem(self.iconList.findItems(icon, Qt.MatchExactly)[0])

    def getCurrentTab(self):
        if self.ctrlTab.currentIndex() != 0:
            self.cGraphicView = self.ctrlTab.currentWidget()
            self.cGraphicScene = self.cGraphicView.scene()
            self.cTab = self.ctrlTab.tabText(self.ctrlTab.currentIndex())
        else:
            self.cGraphicView = None
            self.cGraphicScene = None
            self.cTab = None

        print "Switch to tab: {0}".format(self.cTab)

    def setIconPos(self, *arg):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            if ss[0] == self.currentItem:
                x = self.posXSpin.value()
                y = self.posYSpin.value()

                ss[0].setX(x)
                ss[0].setY(y)
                bttnName = self.bttnDic[ss[0]]
                self.pickerDic[self.cTab][bttnName][1] = (x, y)

    def offsetIconPos(self, direction="up", *arg):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            for s in ss:
                x = s.x()
                y = s.y()
                if direction == "up":
                    s.setY(y-1)
                elif direction == "down":
                    s.setY(y+1)
                elif direction == "left":
                    s.setX(x-1)
                elif direction == "right":
                    s.setX(x+1)

                bttnName = self.bttnDic[s]
                self.pickerDic[self.cTab][bttnName][1] = (s.pos().x(), s.pos().y())

            self.showIconInfo(self.cGraphicScene, self.cTab)

    def alignIconPos(self, direction="up", *arg):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            for i, s in enumerate(ss):
                x = s.x()
                y = s.y()
                if direction == "up":
                    if i == 0:
                        newY = y
                    else:
                        if y < newY: newY = y 
                elif direction == "down":
                    if i == 0:
                        newY = y
                    else:
                        if y > newY: newY = y
                elif direction == "left":
                    if i == 0:
                        newX = x
                    else:
                        if x > newX: newX = x
                elif direction == "right":
                    if i == 0:
                        newX = x
                    else:
                        if x < newX: newX = x

            for s in ss: 
                if direction in ["up", "down"]:
                    s.setY(newY)

                elif direction in ["right", "left"]:
                    s.setX(newX)

    def updateIconCtrls(self):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            if ss[0] == self.currentItem:
                bttnName = self.bttnDic[ss[0]]
                strCtrls = self.ctrlsLine.text()
                ctrls = strCtrls.split(" ") if strCtrls else []
                [ctrls.remove(ctrl) for ctrl in ctrls if not ctrl]
                self.pickerDic[self.cTab][bttnName][2] = ctrls
                self.ctrlPickerDic[ss[0]] = ctrls

    def updateIconName(self):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            if ss[0] == self.currentItem:
                bttnName = self.bttnDic[ss[0]]

                newName = self.pickerNameLine.text()
                if newName not in self.pickerDic[self.cTab].keys():
                    self.bttnDic[ss[0]] = newName
                    self.pickerDic[self.cTab][newName] = list(self.pickerDic[self.cTab][bttnName])
                    self.pickerDic[self.cTab].pop(bttnName, None)

    def updateIconPic(self):
        print ""

    def mirrorPickers(self):
        ss = self.cGraphicScene.selectedItems()
        if ss:
            findName = self.findNameLine.text()
            replaceName = self.replaceNameLine.text()
            findCtrl = self.findCtrlLine.text()
            replaceCtrl = self.replaceCtrlLine.text()

            for s in ss:
                bttnName = self.bttnDic[s]
                if findName in bttnName:
                    newName = bttnName.replace(findName, replaceName)

                    #assign icon
                    icon = self.pickerDic[self.cTab][bttnName][0]
                    iconPath = os.path.join(self.iconDir, "Picker", "{0}.png".format(icon))

                    #create icon
                    bttnItem = self.cGraphicScene.addPixmap(iconPath)
                    bttnItem.setFlag(QGraphicsItem.ItemIsMovable)
                    bttnItem.setFlag(QGraphicsItem.ItemIsSelectable)
                    w = bttnItem.pixmap().size().width()

                    #assign position
                    pos = self.pickerDic[self.cTab][bttnName][1]
                    x = 455-pos[0]-w
                    y = pos[1]
                    bttnItem.moveBy(x, y)
                    print pos
                    print x, y

                    #assign ctrls
                    newCtrls = []
                    for ctrl in self.pickerDic[self.cTab][bttnName][2]:
                        if findCtrl in ctrl:
                            newCtrls.append(ctrl.replace(findCtrl, replaceCtrl))

                    self.pickerDic[self.cTab][newName] = [icon, (x, y), newCtrls]
                    self.ctrlPickerDic[bttnItem] = newCtrls
                    self.bttnDic[bttnItem] = newName

                    print self.pickerDic[self.cTab][newName]

    #--------
    def createNewPickerTab(self):
        newTab = self.newTabLine.text()

        if newTab:
            if newTab not in self.pickerDic.keys():
                self.pickerDic[newTab] = {}

                graphicScene = QGraphicsScene()
                graphicsView = QGraphicsView(graphicScene)
                self.ctrlTab.addTab(graphicsView, newTab)

                graphicsView.setInteractive(True)
                graphicsView.setDragMode(QGraphicsView.RubberBandDrag)
                graphicsView.setSceneRect(0, 0, 450, 510)
                graphicScene.selectionChanged.connect(partial(self.selectPickerCtrls, graphicScene, newTab))

    def createNewPicker(self):
        bttnName = self.newPickerNameLine.text()
        icon = self.newIconList.currentItem().text()
        x = self.newPosXSpin.value()
        y = self.newPosYSpin.value()
        strCtrls = self.newCtrlsLine.text()
        ctrls = strCtrls.split(" ") if strCtrls else []

        if self.cTab not in self.pickerDic.keys():
            self.pickerDic[self.cTab] = {}

        iconPath = os.path.join(self.iconDir, "Picker", "{0}.png".format(icon))

        if os.path.isfile(iconPath):
            self.pickerDic[self.cTab][bttnName] = [icon, (x, y), ctrls]

            bttnItem = self.cGraphicScene.addPixmap(iconPath)
            bttnItem.setFlag(QGraphicsItem.ItemIsMovable)
            bttnItem.setFlag(QGraphicsItem.ItemIsSelectable)
            bttnItem.moveBy(x, y)
            self.ctrlPickerDic[bttnItem] = ctrls
            self.bttnDic[bttnItem] = bttnName

    def loadSelected(self, lineEdit):
        ss = cmds.ls(sl=True)
        if not ss: return False

        strSS = ""
        for s in ss:
            strSS += s + " "

        lineEdit.setText(strSS)

    def deletePickers(self):
        if self.cGraphicScene: 
            ss = self.cGraphicScene.selectedItems()
            if ss:
                for s in ss:
                    self.cGraphicScene.removeItem(s)
                    bttnName = self.bttnDic[s]
                    del self.pickerDic[self.cTab][bttnName]

    def deletePickersTab(self):
        if self.cGraphicScene:
            ss = self.cGraphicScene.items()
            for s in ss:
                if s in self.ctrlPickerDic.keys():
                    del self.ctrlPickerDic[s]
                    del self.bttnDic[s]

            index = self.ctrlTab.currentIndex()
            tab = self.ctrlTab.tabText(index)
            graphicView = self.ctrlTab.currentWidget()
            self.ctrlTab.removeTab(index)
            graphicView.deleteLater()
            del self.pickerDic[tab]

    #-----       
    def updatePickerDic(self):
        """
        update picker dictionary in the current scene file
        """
        #pprint.pprint(self.pickerDic, width=150)

        for i in range(self.ctrlTab.count()-1):
            tab = self.ctrlTab.tabText(i+1)
            graphicView = self.ctrlTab.widget(i+1)
            graphicScene = graphicView.scene()

            ss = graphicScene.items()
            for s in ss:
                #print s.pos()
                if s in self.bttnDic.keys():
                    bttnName = self.bttnDic[s]
                    self.pickerDic[tab][bttnName][1] = (s.pos().x(), s.pos().y())

    def savePickerDicInScene(self):
        self.updatePickerDic()
            
        #create an empty node, add a string attribute and save the data
        pickData = "World"
        if not cmds.objExists("{0}.pickerDic".format(pickData)):
            #cmds.createNode("unknown", n=pickData)
            cmds.addAttr(pickData, ln="pickerDic", dt="string")
        cmds.setAttr("{0}.pickerDic".format(pickData), str(self.pickerDic), type="string")

    def exportPickerDic(self):
        self.updatePickerDic()
        if self.pickerDic:
            self.pickerPath = self.exportPathLine.text()
            fileName = self.exportNameLine.text()
            if not fileName:
                fileName = cmds.file(q=True, sceneName=True, expandName=True).rpartition("/")[2].partition(".")[0]

            self.pickerFilePath = os.path.join(self.pickerPath, "{0}.json".format(fileName))

            overwrite = "Yes"
            if os.path.isfile(self.pickerFilePath):
                overwrite = cmds.confirmDialog(title='Overwrite', message='Do you want to overwrite an existing picker file?', 
                                               button=['Yes','No'], defaultButton='Yes', cancelButton='No', dismissString='No')
            if overwrite == "Yes":
                saveJson(self.pickerDic, self.pickerFilePath)

                print "Data Exported successfully at {0}\n".format(self.pickerPath)
                pprint.pprint(self.pickerDic, width=150)

    def importPickerDic(self):
        try:
            pickerPath = cmds.fileDialog2(dialogStyle=2, dir=self.path, cap="Select Picker Data", okc="Select", fm=1, ff="*.json")[0]
        except:
            pickerPath = None

        if pickerPath:
            print pickerPath        
            self.pickerDic = loadJson(pickerPath)
            self.drawPickerLayout()

    def printPickerDic(self):
        if self.cGraphicScene:
            ss = self.cGraphicScene.selectedItems()
            if ss:
                printDic = {}
                printDic[self.cTab] = {}

                for s in ss:
                    bttnName = self.bttnDic[s]
                    printDic[self.cTab][bttnName] = self.pickerDic[self.cTab][bttnName]

                self.bttnDic
                print "\n"
                pprint.pprint(printDic, width=150)
                return True

        pprint.pprint(self.pickerDic, width=150)
        #print self.ctrlPickerDic
        #print self.bttnDic

    def setNewExportPath(self):
        startDir = self.exportPathLine.text()
        print startDir
        if not os.path.isdir(startDir):
            startDir = cmds.workspace(q=True, rd=True)
        self.pickerPath = cmds.fileDialog2(dialogStyle=2, dir=startDir, cap="Select Folder", okc="Select", fm=3)[0]
        if self.pickerPath[-1] == "/":
                self.pickerPath = self.pickerPath[:-1]

        self.exportPathLine.setText(self.pickerPath.replace("/", os.path.sep))

    def openFileFolder(self):
        filePath = self.exportPathLine.text()
        print filePath
        try:
            os.startfile(filePath)
        except:
            pass

    def closeEvent(self, event):
        super(CharacterTool, self).closeEvent(event)

        self.editPickerWidget.close()
        self.newPickerWidget.close()
        self.mirrorPickerWidget.close()
        self.newTabWidget.close()

